/*	Timer 1 Example Program
	Copyright 1999 Bagotronix Inc.
	Author:  Ivan Baggett

	This program shows how to use Timer 1 to generate a variable duty cycle
	waveform output with interrupt on both the rising edge and trailing edge
	of the waveform.
*/

#include <stdio.h>
#include <dos.h>
#include "dosstamp.h"

#define TMR_RIU	0x1000
#define	TMR_MC	0x0020
#define	TMR_INT	0x2000

/*	Bit masks for PIO configuration.  A 1 in a bit position means that bit
	can be modified.  A 0 means it cannot.

	PIOs 2-9,14,15,22-26,29 are not allowed to be changed.  They are used
	by critical functions.
*/
UWORD piomodes[32] = {
	0x0001,	/* PIO0 */
	0x0002,	/* PIO1 */
	0x0000,	/* PIO2, not available */
	0x0000,	/* PIO3, not available */
	0x0000,	/* PIO4, not available */
	0x0000,	/* PIO5, not available */
	0x0000,	/* PIO6, not available */
	0x0000,	/* PIO7, not available */
	0x0000,	/* PIO8, not available */
	0x0000,	/* PIO9, not available */
	0x0400,	/* PIO10 */
	0x0800,	/* PIO11 */
	0x1000,	/* PIO12 */
	0x2000,	/* PIO13 */
	0x0000,	/* PIO14, not available */
	0x0000,	/* PIO15, not available */

	0x0001,	/* PIO16 */
	0x0002,	/* PIO17 */
	0x0004,	/* PIO18 */
	0x0008,	/* PIO19 */
	0x0010,	/* PIO20 */
	0x0020,	/* PIO21 */
	0x0000,	/* PIO22, not available */
	0x0000,	/* PIO23, not available */
	0x0000,	/* PIO24, not available */
	0x0000,	/* PIO25, not available */
	0x0000,	/* PIO26, not available */
	0x0800,	/* PIO27 */
	0x1000,	/* PIO28 */
	0x2000,	/* PIO29, not available */
	0x4000,	/* PIO30 */
	0x8000	/* PIO31 */
};



void interrupt (*oldvect)();
ULONG aaa, bbb;

void interrupt timer1_isr (void) {
	// increment event counter
	if (inport (TMR1_CON) & TMR_RIU)
		++aaa;
	else
		++bbb;

	// reset the MC bit
	outport (TMR1_CON, inport (TMR1_CON) & (~TMR_MC));

	// not a typo; all timers use same EOI type as Timer 0, even though they
	// have their own vectors
	SEOI(TMR0_VEC);
}

int main (void) {
	UWORD aa, bb;
	ULONG a, b;
	BOOLEAN done = FALSE;
	WORD c;
	BYTE s[128];

	disable();
	oldvect = getvect (TMR1_VEC);
	setvect (TMR1_VEC, timer1_isr);
	enable();

	// configure PIO1 as Timer 1 output
	PIOconfig (1, PIOCFG_NORMAL, 0);

	a = 0L;
	b = 0L;
	aaa = 0L;
	bbb = 0L;

	// Timer interrupts are already enabled in BIOS for Timer 0
	// So no interrupt controller stuff is needed

	// initialize timer 1 with a 50% duty cycle
	outport (TMR1_CMPA, 0x1000);
	outport (TMR1_CMPB, 0x1000);
	outport (TMR1_COUNT, 0);
	// enabled, interrupt enabled, no retrig, prescaled, internal clock, alternate compare, continuous counting
	outport (TMR1_CON, 0xe00b);


	printf ("\nDOS Stamp Timer 1 variable duty cycle demonstration");
	printf ("\nObserve the waveform output on JP1 pin 19 (TMROUT1/PIO1/GPIO7).");
	printf ("\nHit 'X' to quit, other key to enter a new duty cycle...");

	while (!done) {
		if (kbhit()) {
			c = getch();
			if (!c)
				c = getch();
			if (c == 'X' || c == 'x')
				done = TRUE;
			else {
				disable();
				a = aaa;
				b = bbb;
				enable();
				printf ("\nInterrupt events for A = %lu", a);
				printf ("\nInterrupt events for B = %lu", b);

				printf ("\nEnter Max Count A value in hex: ");
				gets (s);
				sscanf (s, "%x", &aa);
				printf ("\nEnter Max Count B value in hex: ");
				gets (s);
				sscanf (s, "%x", &bb);
				printf ("\nA = %x, B = %x", aa, bb);

				// change max count registers
				while (!(inport (TMR1_CON) & TMR_RIU));	// wait for B
				while (inport (TMR1_CON) & TMR_RIU);	// wait for A
				outport (TMR1_CMPB, bb);				// program B

				while (inport (TMR1_CON) & TMR_RIU);	// wait for A
				while (!(inport (TMR1_CON) & TMR_RIU));	// wait for B
				outport (TMR1_CMPA, aa);				// program A
			}
		}
	}

	// deactivate timer 1 interrupts
	outport (TMR1_CON, inport (TMR1_CON) & ~(TMR_INT));

	disable();
	setvect (TMR1_VEC, oldvect);
	enable();

	return 0;
}


/****************************************************************************
PIOconfig():  configures a PIO pin

	arguments:
		1) PIO number (0 - 31)
		2) PIO configuration (0 - 3)
		3) PIO pin initial value (0 = low, 1 = high)
	returns:  nothing
	resources used:  the PIO pin specified, PIO control registers
	comments:
		Interrupts are not disabled during this function.  You may need to
		disable them before calling this function and reenable them
		afterwards.
	verified:
****************************************************************************/
void PIOconfig (UWORD pionum, UWORD piocfg, BOOLEAN initval) {
	UWORD bitmask;
	UWORD temp;
	UWORD ofs;

	if (pionum < sizeof piomodes / sizeof (UWORD)) { 	// check bounds
		ofs = pionum / 16;		// select the PIO register set (0 or 1)
		ofs = ofs ? 6 : 0;	// convert to an offset
		bitmask = piomodes[pionum];	// get PIO bit mask

		// write the PIO data
		temp = inport (PIO0_DATA + ofs);
		outport (PIO0_DATA + ofs,
			(temp & ~bitmask) | (initval ? bitmask : 0));

		// write the PIO mode
		temp = inport (PIO0_MODE + ofs);
		outport (PIO0_MODE + ofs,
			(temp & ~bitmask) | ((piocfg & 0x0002) ? bitmask : 0));

		// write the PIO direction
		temp = inport (PIO0_DIR + ofs);
		outport (PIO0_DIR + ofs,
			(temp & ~bitmask) | ((piocfg & 0x0001) ? bitmask : 0));
	}
}


